package com.gic.store.service.outer;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.entity.TabStoreFieldSelect;
import com.gic.store.service.StoreFieldSelectApiService;
import com.gic.store.service.StoreFieldSelectService;
import com.gic.store.utils.ErrorCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author guojx
 * @date 2019/7/1 2:52 PM
 */
@Service("storeFieldSelectApiService")
public class StoreFieldSelectApiServiceImpl implements StoreFieldSelectApiService{
    @Autowired
    private StoreFieldSelectService storeFieldSelectService;

    @Override
    public ServiceResponse<Integer> save(StoreFieldSelectDTO storeFieldSelectDTO) {
        if (StringUtils.isBlank(storeFieldSelectDTO.getStoreFieldSelectName())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值名称不能为空");
        }
        if (StringUtils.isBlank(storeFieldSelectDTO.getStoreFieldSelectCode())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值code不能为空");
        }
        if (storeFieldSelectDTO.getStoreFieldId() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "自定义字段值不能为空");
        }
        if (storeFieldSelectDTO.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "企业ID不能为空");
        }
        if (storeFieldSelectService.isRepeatCode(storeFieldSelectDTO.getStoreFieldId(), null, storeFieldSelectDTO.getStoreFieldSelectCode())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值code不能重复");
        }

        if (storeFieldSelectService.isRepeatName(storeFieldSelectDTO.getStoreFieldId(), null, storeFieldSelectDTO.getStoreFieldSelectName())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值名称不能重复");
        }

        storeFieldSelectDTO.setCreateTime(new Date());
        storeFieldSelectDTO.setUpdateTime(new Date());
        storeFieldSelectDTO.setSort(storeFieldSelectService.getMinSort(storeFieldSelectDTO.getStoreFieldId()) - 1);

        return ServiceResponse.success(storeFieldSelectService.save(storeFieldSelectDTO));
    }

    @Override
    public ServiceResponse<Integer> upOrDown(int storeFieldSelectId, boolean isUp) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        TabStoreFieldSelect needUpdateSort;
        if (isUp) {
            needUpdateSort = storeFieldSelectService.getPrevSort(tabStoreFieldSelect.getStoreFieldId(), tabStoreFieldSelect.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "已经是第一条，不能上移");
            }
        } else {
            needUpdateSort = storeFieldSelectService.getNextSort(tabStoreFieldSelect.getStoreFieldId(), tabStoreFieldSelect.getSort());
            if (needUpdateSort == null) {
                return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "已经是最后一条，不能下移");
            }
        }
        storeFieldSelectService.updateSortById(needUpdateSort.getStoreFieldSelectId(), tabStoreFieldSelect.getSort());
        storeFieldSelectService.updateSortById(tabStoreFieldSelect.getStoreFieldSelectId(), needUpdateSort.getSort());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> setStoreFieldSelectSort(int storeFieldSelectId, int setValue) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        storeFieldSelectService.setStoreFieldSelectSort(storeFieldSelectId, tabStoreFieldSelect.getStoreFieldId(), setValue);
        return ServiceResponse.success(1);
    }

    @Override
    public ServiceResponse<Integer> delete(int storeFieldSelectId) {
        return ServiceResponse.success(storeFieldSelectService.delete(storeFieldSelectId));
    }

    @Override
    public ServiceResponse<Integer> edit(StoreFieldSelectDTO storeFieldSelectDTO) {
        if (StringUtils.isBlank(storeFieldSelectDTO.getStoreFieldSelectName())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值名称不能为空");
        }
        if (StringUtils.isBlank(storeFieldSelectDTO.getStoreFieldSelectCode())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值code不能为空");
        }
        if (storeFieldSelectDTO.getStoreFieldId() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键ID不能为空");
        }

        if (storeFieldSelectService.isRepeatCode(storeFieldSelectDTO.getStoreFieldId(), storeFieldSelectDTO.getStoreFieldSelectId(), storeFieldSelectDTO.getStoreFieldSelectCode())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值code不能重复");
        }

        if (storeFieldSelectService.isRepeatName(storeFieldSelectDTO.getStoreFieldId(), storeFieldSelectDTO.getStoreFieldSelectId(), storeFieldSelectDTO.getStoreFieldSelectName())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "属性值名称不能重复");
        }

        return ServiceResponse.success(storeFieldSelectService.edit(storeFieldSelectDTO.getStoreFieldSelectId(),
                storeFieldSelectDTO.getStoreFieldSelectName(), storeFieldSelectDTO.getStoreFieldSelectCode()));
    }

    @Override
    public ServiceResponse<Page<StoreFieldSelectDTO>> listStoreFieldSelect(int storeFieldId, String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page listStoreFieldSelect = storeFieldSelectService.listStoreFieldSelect(storeFieldId, search, pageNum, pageSize);
        Page<StoreFieldSelectDTO> selectDTOPage = PageHelperUtils.changePageHelperToCurrentPage(listStoreFieldSelect, StoreFieldSelectDTO.class);
        List<StoreFieldSelectDTO> selectDTOList = selectDTOPage.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0, length = selectDTOList.size(); i < length; i++) {
            selectDTOList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(selectDTOPage);
    }

    @Override
    public ServiceResponse<StoreFieldSelectDTO> getStoreFieldSelect(int storeFieldSelectId) {
        return ServiceResponse.success(EntityUtil.changeEntityNew(StoreFieldSelectDTO.class, storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId)));
    }

    @Override
    public ServiceResponse<Integer> sortBottom(int storeFieldSelectId) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        storeFieldSelectService.sortBottom(tabStoreFieldSelect.getStoreFieldId(), storeFieldSelectId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> sortTopping(int storeFieldSelectId) {
        TabStoreFieldSelect tabStoreFieldSelect = storeFieldSelectService.getStoreFieldSelectById(storeFieldSelectId);
        if (tabStoreFieldSelect == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        storeFieldSelectService.sortTopping(tabStoreFieldSelect.getStoreFieldId(), storeFieldSelectId);
        return ServiceResponse.success();
    }

}
